import json
import urllib2
import ssl

if hasattr(ssl, 'create_default_context'):
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE
else:
    ctx = None

from django.core.management.base import BaseCommand, CommandError
import django.utils.timezone

from scrutiny.projects.models import InstalledService, Module, Version, InstalledVersion

class Command(BaseCommand):
    help = 'Scrutiny services for changes'

    def handle(self, verbosity, *args, **options):
        self.verbose = (verbosity > 1)
        for service in InstalledService.objects.all():
            if not service.url.endswith('/'):
                service.url = '%s/' % service.url
            if self.verbose:
                print 'Checking', service.url
            try:
                if ctx:
                    versions = json.load(urllib2.urlopen('%s__version__' % service.url,
                            timeout=5, context=ctx))
                else:
                    versions = json.load(urllib2.urlopen('%s__version__' % service.url,
                            timeout=5))
            except urllib2.HTTPError, e:
                print 'Error with', service.url, e
                continue
            except urllib2.URLError, e:
                print 'Error with', service.url, e
                continue

            current_modules = set([x for x in
                service.service.get_modules(platforms=[service.platform])])
            seen_modules = set()

            for module_name, version_string in versions.items():
                try:
                    module = Module.objects.get(name=module_name)
                except Module.DoesNotExist:
                    module = Module()
                    module.name = module_name
                    module.save()

                seen_modules.add(module)
                try:
                    version = Version.objects.get(module=module, version=version_string)
                except Version.DoesNotExist:
                    version = module.version_set.create(version=version_string)

                try:
                    installed_version = InstalledVersion.objects.get(
                            service=service, version=version)
                except InstalledVersion.DoesNotExist:
                    installed_version = InstalledVersion()
                    installed_version.service = service
                    installed_version.version = version
                    installed_version.timestamp = django.utils.timezone.now()
                    installed_version.save()

            uninstalled_modules = current_modules - seen_modules
            for module in uninstalled_modules:
                try:
                    version = Version.objects.get(module=module, version='')
                except Version.DoesNotExist:
                    version = module.version_set.create(version='')

                installed_version = InstalledVersion()
                installed_version.service = service
                installed_version.version = version
                installed_version.timestamp = django.utils.timezone.now()
                installed_version.save()
