import json
import re
import threading
import time
import urllib2

from django.conf import settings
from django.core.cache import cache

CACHE_DURATION = 3600


def decorate_commit_line(line):
    escaped_line = line.replace('<', '&lt;').replace('>', '&gt;')
    if line.startswith('<'):
        return '<span class="in">%s</span>' % escaped_line
    elif line.startswith('>'):
        return '<span class="out">%s</span>' % escaped_line
    return '<span class="other">%s</span>' % escaped_line


class Issue(object):
    def __init__(self, issue_id):
        self.id = issue_id
        self.commits = []

    def update_content(self):
        request = urllib2.Request(
                '%s/issues/%s.json' % (settings.REDMINE_URL, self.id),
                headers={'X-Redmine-API-Key': settings.REDMINE_API_KEY})
        f = urllib2.urlopen(request)
        content = json.loads(f.read())
        cache.set(self.id, content, CACHE_DURATION)

    def get_attribute(self, attr):
        content = cache.get(self.id)
        if content:
            return content.get('issue').get(attr)
        try:
            self.update_content()
        except urllib2.HTTPError:
            return '---'
        return self.get_attribute(attr)

    @property
    def subject(self):
        return self.get_attribute('subject')

    @property
    def url(self):
        return '%s/issues/%s' % (settings.REDMINE_URL, self.id)

    def add_commit(self, commit):
        self.commits.append(commit)


class CommitAndIssues(object):

    def __init__(self, oneline):
        self.issues = []
        self.commit = oneline.strip()
        self.commit_html = decorate_commit_line(self.commit)

    @classmethod
    def get_for_commits(cls, module, v1, v2):
        difflog = module.get_diff_log(v1, v2, format='full', grep=r'\#[0-9]\+')
        commits = re.findall(r'^commit.*?\n\n.*?\n\n', difflog, re.DOTALL | re.MULTILINE)
        oneline_commits = module.get_diff_log(v1, v2, grep=r'\#[0-9]\+').splitlines()

        objects = []
        for commit, oneline in zip(commits, oneline_commits):
            obj = cls(oneline.strip())
            for issue_id in re.findall('#(\d+)', commit):
                obj.add_issue(issue_id)
            objects.append(obj)

        return objects

    def add_issue(self, issue_id):
        self.issues.append(Issue(issue_id))
